import os
from pathlib import Path
import re
import subprocess
import dbus
from .log_utils import get_logger

#dm_service_path = ""
#init_system = ""
#openrc_pid = 0
#runit_pid = 0
#s6_pid = 0

class CheckError(Exception):
    pass


def check_running_graphical_session():
    return subprocess.run(
        "xhost",
        shell=True,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL
    ).returncode == 0


def is_ac_power_connected():

    for power_source_path in Path("/sys/class/power_supply/").iterdir():

        try:

            with open(power_source_path / "type", 'r') as f:
                if f.read().strip() != "Mains":
                    continue

            with open(power_source_path / "online", 'r') as f:
                if f.read(1) == "1":
                    return True

        except IOError:
            continue

    return False


def is_pat_available():
    return subprocess.run(
        "grep -E '^flags.+ pat( |$)' /proc/cpuinfo",
        shell=True, stdout=subprocess.DEVNULL
    ).returncode == 0


def get_active_renderer():

    if _is_gl_provider_nvidia():
        return "nvidia"
    else:
        return "integrated"


def is_module_available(module_name):

    return subprocess.run(
        f"modinfo {module_name}",
        shell=True, stdout=subprocess.DEVNULL
    ).returncode == 0

def is_module_loaded(module_name):

    return subprocess.run(
        f"lsmod | grep -E \"^{module_name}\"",
        shell=True, stdout=subprocess.DEVNULL
    ).returncode == 0


def get_current_display_manager():
	
	openrc_pid_subprocess = subprocess.Popen("pidof init", shell=True, stdout=subprocess.PIPE)
	openrc_pid_return = openrc_pid_subprocess.stdout.read()

	if openrc_pid_return == b'1\n':
		openrc_pid = 1 
	
	runit_pid_subprocess = subprocess.Popen("pidof runit", shell=True, stdout=subprocess.PIPE)
	runit_pid_return = runit_pid_subprocess.stdout.read()

	if runit_pid_return == b'1\n':
		runit_pid = 1 
	
	s6_pid_subprocess = subprocess.Popen("pidof s6-svscan", shell=True, stdout=subprocess.PIPE)
	s6_pid_return = s6_pid_subprocess.stdout.read()

	if s6_pid_return == b'1\n':
		s6_pid = 1 
	
	if (openrc_pid == 0) and (runit_pid == 0) and (s6_pid == 0):
		print("Not an Artix Linux flavour, the daemon stops")
		sys.exit(1)
	
			
	if openrc_pid == 1:
		init_system = "openrc"
	if runit_pid  == 1:
		init_system = "runit"
	if s6_pid     == 1:
		init_system = "s6"
			
	##### OpenRC check for DM service #####
		
	if init_system == "openrc" and os.path.isfile("/run/openrc/daemons/gdm/001"):
		dm_service_path = os.path.realpath("/etc/init.d/gdm")
		
	if init_system == "openrc" and os.path.isfile("/run/openrc/daemons/lightdm/001"):
		dm_service_path = os.path.realpath("/etc/init.d/lightdm")
		
	if init_system == "openrc" and os.path.isfile("/run/openrc/daemons/lxdm/001"):
		dm_service_path = os.path.realpath("/etc/init.d/lxdm")
		
	if init_system == "openrc" and os.path.isfile("/run/openrc/daemons/sddm/001"):
		dm_service_path = os.path.realpath("/etc/init.d/sddm")

	if init_system == "openrc" and os.path.isfile("/run/openrc/daemons/xdm/001"):
		dm_service_path = os.path.realpath("/etc/init.d/xdm")

	##### Runit check for DM service  #####	

	if init_system == "runit" and os.path.isfile("/run/runit/service/gdm/run"):
		dm_service_path = os.path.realpath("/etc/runit/sv/gdm")
		
	if init_system == "runit" and os.path.isfile("/run/runit/service/lightdm/run"):
		dm_service_path = os.path.realpath("/etc/runit/sv/lightdm")
		
	if init_system == "runit" and os.path.isfile("/run/runit/service/lxdm/run"):
		dm_service_path = os.path.realpath("/etc/runit/sv/lxdm")
		
	if init_system == "runit" and os.path.isfile("/run/runit/service/sddm/run"):
		dm_service_path = os.path.realpath("/etc/runit/sv/sddm")

	if init_system == "runit" and os.path.isfile("/run/runit/service/xdm/run"):
		dm_service_path = os.path.realpath("/etc/runit/sv/xdm")

	##### s6 check for DM service  #####
	
	if init_system == "s6" and os.path.isfile("/run/s6/service/gdm/run"):
		dm_service_path = os.path.realpath("/etc/s6/sv/gdm")
		
	if init_system == "s6" and os.path.isfile("/run/s6/service/lightdm/run"):
		dm_service_path = os.path.realpath("/etc/s6/sv/lightdm")
		
	if init_system == "s6" and os.path.isfile("/run/s6/service/lxdm/run"):
		dm_service_path = os.path.realpath("/etc/s6/sv/lxdm")
		
	if init_system == "s6" and os.path.isfile("/run/s6/service/sddm/run"):
		dm_service_path = os.path.realpath("/etc/s6/sv/sddm")

	if init_system == "s6" and os.path.isfile("/run/s6/service/xdm/run"):
		dm_service_path = os.path.realpath("/etc/s6/sv/xdm")

	if dm_service_path == "":
		print("No display-manager service file found, the daemon stops")
		sys.exit(1)

	dm_service_filename = os.path.split(dm_service_path)[-1]
	dm_name = os.path.splitext(dm_service_filename)[0]

	return dm_name


def using_patched_GDM():

    folder_path_1 = "/etc/gdm/Prime"
    folder_path_2 = "/etc/gdm3/Prime"

    return os.path.isdir(folder_path_1) or os.path.isdir(folder_path_2)

def check_offloading_available():

    try:
        out = subprocess.check_output(
            "xrandr --listproviders", shell=True, text=True, stderr=subprocess.PIPE).strip()
    except subprocess.CalledProcessError as e:
        raise CheckError(f"Cannot list xrandr providers:\n{e.stderr}") from e

    for line in out.splitlines():
        if re.search("^Provider [0-9]+:", line) and "name:NVIDIA-G0" in line:
            return True
    return False


def is_xorg_intel_module_available():
    return os.path.isfile("/usr/lib/xorg/modules/drivers/intel_drv.so")

def is_xorg_amdgpu_module_available():
    return os.path.isfile("/usr/lib/xorg/modules/drivers/amdgpu_drv.so")


def is_login_manager_active():
    return _is_service_active("display-manager")


def is_daemon_active():
    return _is_service_active("optimus-manager")


def is_bumblebeed_service_active():
    return _is_service_active("bumblebeed")


def _is_gl_provider_nvidia():

    try:
        out = subprocess.check_output(
            "__NV_PRIME_RENDER_OFFLOAD=0 glxinfo",
            shell=True, text=True, stderr=subprocess.PIPE).strip()
    except subprocess.CalledProcessError as e:
        raise CheckError(f"Cannot run glxinfo: {e.stderr}") from e

    for line in out.splitlines():
        if "server glx vendor string: NVIDIA Corporation" in line:
            return True
    return False


#	dm_service_path = ""
#	init_system = ""
#	openrc_pid = 0
#	runit_pid = 0
#	s6_pid = 0

def _is_service_active(service_name):

    logger = get_logger()

    try:
        system_bus = dbus.SystemBus()
    except dbus.exceptions.DBusException:
        logger.warning(
            "Cannot communicate with the DBus system bus to check status of %s."
            " Is DBus running ? Falling back to bash commands", service_name)
        return _is_service_active_bash(service_name)
    else:
        return _is_service_active_dbus(system_bus, service_name)
    #   return _is_service_active_bash(service_name)

def _is_service_active_dbus(system_bus, service_name):

    systemd = system_bus.get_object("org.freedesktop.login1", "/org/freedesktop/login1")

    try:
        unit_path = systemd.GetUnit("%s.service" % service_name, dbus_interface="org.freedesktop.login1.Manager")
    except dbus.exceptions.DBusException:
        return False

    optimus_manager_interface = system_bus.get_object("org.freedesktop.login1", unit_path)
    properties_manager = dbus.Interface(optimus_manager_interface, 'org.freedesktop.DBus.Properties')
    state = properties_manager.Get("org.freedesktop.login1.Unit", "SubState")

    return state == "running"


def _is_service_active_bash(service_name):

    service_check=""

    service_check = subprocess.run(
        f"pidof {service_name}", shell=True
    ).returncode == 0

    if service_check != 0:
        return("active")
